import os
import time
import random
import logging
import re
from pathlib import Path
from queue import Queue
import requests
from bs4 import BeautifulSoup

# 日志配置
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
)

class APIController:
    """API速率控制核心类"""
    def __init__(self, api_key, max_workers=5, retries=3, base_delay=5):
        self.api_key = api_key
        self.max_workers = max_workers  # 并发数提高到5
        self.retries = retries          # 重试次数增加到3
        self.base_delay = base_delay    # 基础退避时间(秒)
        self.req_counter = 0           # 请求计数器
        self.last_reset = time.time()   # 周期计时器

    def make_request(self, prompt, max_tokens=2000):
        """增强版请求方法"""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json"
        }
        payload = {
            "model": "deepseek-chat",
            "messages": [{"role": "user", "content": prompt}],
            "temperature": 1.3,
            "max_tokens": max_tokens
        }

        for attempt in range(self.retries + 1):
            try:
                # 动态速率控制
                self._rate_limit()
                
                response = requests.post(
                    "https://api.deepseek.com/v1/chat/completions",
                    headers=headers,
                    json=payload,
                    timeout=40
                )
                response.raise_for_status()
                data = response.json()
                return data['choices'][0]['message']['content']
                
            except requests.exceptions.HTTPError as e:
                if e.response.status_code in [429, 500, 502, 503, 504]:  # 扩展可重试状态码
                    delay = self._calculate_backoff(attempt)
                    logging.warning(f"HTTP错误 {e.response.status_code}，等待 {delay}秒后重试 (尝试 {attempt + 1}/{self.retries})")
                    time.sleep(delay)
                    continue
                logging.error(f"API请求失败: {str(e)}")
                return None
            except (requests.exceptions.Timeout, requests.exceptions.ConnectionError) as e:
                delay = self._calculate_backoff(attempt)
                logging.warning(f"网络异常 {type(e).__name__}，等待 {delay}秒后重试 (尝试 {attempt + 1}/{self.retries})")
                time.sleep(delay)
                continue
            except Exception as e:
                logging.error(f"请求异常: {str(e)}")
                if attempt == self.retries:
                    return None
                delay = self._calculate_backoff(attempt)
                time.sleep(delay)
        return None

    def _rate_limit(self):
        """动态速率控制策略"""
        # 每60秒最多(max_workers*20)次请求
        if time.time() - self.last_reset > 60:
            self.req_counter = 0
            self.last_reset = time.time()
        
        if self.req_counter >= self.max_workers * 20:
            sleep_time = 60 - (time.time() - self.last_reset)
            logging.info(f"周期请求数已达上限，等待 {sleep_time:.1f}秒")
            time.sleep(sleep_time)
            self.last_reset = time.time()
            self.req_counter = 0
        
        self.req_counter += 1
        time.sleep(random.uniform(0.1, 0.5))  # 添加随机抖动

    def _calculate_backoff(self, attempt):
        """指数退避算法"""
        return min(self.base_delay * (2 ** attempt), 60) + random.uniform(0, 1)

class BatchProcessor:
    """增强版批量处理器"""
    def __init__(self, api_controller, save_dir, max_failures=10, max_retries=3):
        self.api_controller = api_controller
        self.save_dir = Path(save_dir)
        self.task_queue = Queue()
        self.failure_queue = Queue()
        self.max_failures = max_failures
        self.max_retries = max_retries  # 每个任务最大重试次数
        self.retry_counts = {}          # 任务重试计数器

    def add_tasks(self, keywords):
        """添加任务到队列"""
        for kw in keywords:
            self.task_queue.put(kw)
            self.retry_counts[kw] = 0  # 初始化重试计数器

    def process_batch(self):
        """处理批量任务"""
        success_count = 0
        total = self.task_queue.qsize()
        processed = 0

        while not self.task_queue.empty() and self.failure_queue.qsize() < self.max_failures:
            keyword = self.task_queue.get()
            processed += 1
            
            try:
                start_time = time.time()
                logging.info(f"处理中 [{processed}/{total}]：{keyword}")
                
                # 生成文章
                prompt = self._build_prompt(keyword)
                content = self.api_controller.make_request(prompt)
                if not content:
                    raise ValueError("生成失败")
                
                # 获取图片
                image_url = self._fetch_image(keyword)
                if image_url:
                    content = self._insert_image(content, image_url, keyword)
                
                # 保存结果
                filepath = self._save_content(content, keyword)
                elapsed = time.time() - start_time
                
                logging.info(f"已完成 [{processed}/{total}] 耗时：{elapsed:.2f}s")
                success_count += 1
                
            except Exception as e:
                logging.error(f"处理失败: {keyword} - {str(e)}")
                if self.retry_counts[keyword] < self.max_retries:
                    self.retry_counts[keyword] += 1
                    self.task_queue.put(keyword)  # 重新加入队列
                    logging.info(f"任务 {keyword} 进入重试 ({self.retry_counts[keyword]}/{self.max_retries})")
                else:
                    self.failure_queue.put(keyword)
                continue

        logging.info(f"批量处理完成，成功率：{success_count}/{total}")
        return success_count

    def _build_prompt(self, keyword):
        return f"""写一篇关于{keyword}的文章。要求：
        - 包含一个h1标题，标题示例：客流监控如何解决店铺管理难题？店铺客流监控解决方法
        - 包含至少3个小节
        - 增加一些口语化的表达
        - 加入一些个人观点或感受
        - 使用第一人称叙述where合适
        - 增加一些修辞手法,如比喻、拟人等
        - 在适当地方加入情感词汇
        - 描述时增加感官细节
        - 使用更自然的过渡词
        - 减少重复的句式结构
        - 不使用"首先、其次、再次、最后"等明显的排序词
        - 避免使用"让我们、我们来"等机械化的引导词
        - 避免使用"总之、综上所述"等明显的总结词
        - 段落全部用<p>标签,主标题用<h1>标签，章节标题用<h3>标签
        - 不要html框架代码和其他符号，文件保存编码为utf-8"""

    def _fetch_image(self, keyword):
        """获取必应图片"""
        try:
            url = "https://www.bing.com/images/async"
            params = {
                "q": keyword,
                "first": 0,
                "count": 20,
                "qft": "+filterui:imagesize-large"
            }
            headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36"}
            
            response = requests.get(url, params=params, headers=headers, timeout=10)
            if response.status_code == 200:
                soup = BeautifulSoup(response.text, 'html.parser')
                images = [img.get('data-src') or img.get('src') 
                         for img in soup.select('img.mimg')[:20]
                         if not (img.get('src') or '').startswith('data:image')]
                return random.choice(images) if images else None
            return None
        except Exception as e:
            logging.error(f"图片获取失败: {str(e)}")
            return None

    def _insert_image(self, content, image_url, keyword):
        """插入图片"""
        first_h3_end = content.find("</h3>")
        if first_h3_end == -1:
            return content
        second_h3_start = content.find("<h3>", first_h3_end)
        if second_h3_start == -1:
            return content
        return content[:second_h3_start] + f'<p><img src="{image_url}" alt="{keyword}"></p>' + content[second_h3_start:]

    def _process_content(self, content):
        """处理内容并提取标题"""
        h1_start = content.find("<h1>")
        h1_end = content.find("</h1>", h1_start) if h1_start != -1 else -1
        
        h1_title = None
        if h1_start != -1 and h1_end != -1:
            h1_title = content[h1_start+4:h1_end].strip()
            h1_title = re.sub(r'[^\w\u4e00-\u9fa5？?]', '', h1_title)
            
            title_replacements = {
                "揭秘": "", "全解析": "", "解析": "", "分析": "",
                "深入": "", "全面": "", "指南": "", "一文": "",
                "读懂": "", "讲清": "",
            }
            for old, new in title_replacements.items():
                h1_title = h1_title.replace(old, new)
            
            clean_content = content[:h1_start] + content[h1_end+5:]
        else:
            clean_content = content
        
        replacements = {
            "总之，": "", "总得来说，": "", "说到底，": "",
            "总的来说，": "", "总结来说，": "", "总结来看，": "",
            "总结：": "", "用大白话": "",
        }
        for old, new in replacements.items():
            clean_content = clean_content.replace(old, new)
        
        return h1_title, clean_content

    def _save_content(self, content, keyword):
        """保存内容到文件"""
        self.save_dir.mkdir(exist_ok=True, parents=True)
        
        h1_title, clean_content = self._process_content(content)
        
        filename = h1_title if h1_title else keyword[:50]
        filename = re.sub(r'[^\w\u4e00-\u9fa5?]', '？', filename)[:100]
        
        filepath = self.save_dir / f"{filename}.txt"
        counter = 1
        while filepath.exists():
            filepath = self.save_dir / f"{filename}_{counter}.txt"
            counter += 1
        
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write(clean_content)
        return filepath

def main():
    """主执行函数"""
    API_KEY = "" # 替换为你的API密钥
    SAVE_DIR = "D:/work/txt_play"
    
    # 初始化控制器
    api_ctl = APIController(
        api_key=API_KEY,
        max_workers=5,    # 并发数提高到5
        retries=3,        # 重试次数增加到3
        base_delay=5
    )
    
    processor = BatchProcessor(
        api_controller=api_ctl,
        save_dir=SAVE_DIR,
        max_failures=50,
        max_retries=3     # 每个任务最多重试3次
    )

    # 获取关键词
    print("输入关键词（每行一个，空行结束）：")
    keywords = []
    while True:
        line = input().strip()
        if not line:
            break
        keywords.append(line)
    
    if not keywords:
        print("未输入有效关键词")
        return

    processor.add_tasks(keywords)
    processor.process_batch()

if __name__ == "__main__":
    main()